package uk.ac.umist.co.brailletrans.tests;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import uk.ac.umist.co.brailletrans.*;
import uk.ac.umist.co.brailletrans.tests.Stopwatch;


public class TranslateC extends Translator
{

  private static final int DLL_ERROR = 4;

  // native function prototypes
  public native int InitBrl(String languageName);
  public native int Translate(byte[] StrIn, byte[] StrOut);
  public native int SetState(byte newState);


  // dll loader - dll should contain all of C functions detailed above
  static
  {
    try
    {
      System.loadLibrary("CBrailleTrans");
    }
    catch(UnsatisfiedLinkError e)
    {
      System.err.println("Could not load CBrailleTrans dll:" + e);
      System.exit(DISK_ERROR);
    }
  } // end of static block


  public static void main(String[] args)
  {
    String inFilename = "";
    String languageToUse = "";
    int[] input = null;
    byte state = 1;
    Stopwatch toLoad = new Stopwatch();
    Stopwatch toRun = new Stopwatch();
    TranslateC wrapper = new TranslateC(); // reference to the C dll
    int resultLoad;
    int resultSet;
    int resultRun;
    int number;


    if (args.length != 4)
    {
      System.out.println("TranslateC");
      System.out.println("USAGE  java TranslateC <file> <language> <state> <number>");
      System.exit(SUCCESS);
    }

    inFilename = new String(args[0]);
    languageToUse = new String(args[1] + Language.FILE_EXTENSION_DELIMITER
      + Language256.FILENAME_EXTENSION);
    state = Byte.parseByte(args[2]);
    number = Integer.parseInt(args[3]);
    args = null;

    try
    {
      System.out.println("Loading " + languageToUse);
      toLoad.start();
      resultLoad = wrapper.InitBrl(languageToUse); // loads language
      toLoad.stop();
    }
    catch (UnsatisfiedLinkError e)
    {
      System.out.println("Failed to link dll for language input with InitBrl:" + e);
      System.exit(DLL_ERROR);
    }

    resultSet = wrapper.SetState(state);

    // Get the input to translate
    try {
      input = Translator.readIntArrayFromDisk(inFilename);
    } catch (Exception e) { System.err.println(e); System.exit(DISK_ERROR); }

    byte[] toConvert = turnIntoByteArray(input);
//    byte[] output = new byte[toConvert.length];


//    System.out.println("Input:<" + Translator.turnIntoString(toConvert) + ">");
    toRun.start();
    for (int i = 0; i < number; i++)
    {
      byte[] toTranslate = new byte[toConvert.length];
      for (int j = 0; j < toTranslate.length; j++)
        toTranslate[j] = toConvert[j];
      byte[] output = new byte[(int) (toConvert.length * Language256.MAX_COMPRESS)];
      resultRun = wrapper.Translate(toTranslate, output); // should translate
//      System.out.println("Output:<" + Translator.turnIntoString(output) + ">");
//      System.err.println("Did " + i);
    }
    toRun.stop();
//    System.out.println("Input:<" + Translator.turnIntoString(toConvert) + ">");

//    System.out.println("Result:" + Translator.turnIntoString(output));
    System.out.println("Time to load: " + toLoad.getTime());
    System.out.println("Time to run: " + toRun.getTime());
  }
}
